///////////////////////////////////////////////////////////////////////////////
/*
                                 display.js

COPYRIGHT
---------
Copyright (C) 2017 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.
http://www.gnu.org/licenses/gpl.txt


VERSION
-------
30-JAN-2017  MGD  initial
*/
///////////////////////////////////////////////////////////////////////////////

/////////////////////////////
// client "debugging" data //
/////////////////////////////

/*
*/

function intrspInsightClick ()
{
   var div = $byId('insight');
   if (intrsp_InsightDisplay)
   {
      intrsp_InsightDisplay = false;
      div.style.display = 'none';
   }
   else
   {
      intrsp_InsightDisplay = true;
      div.style.display = 'block';
   }
   acmeAdjustSize();
}

/*
*/

var intrsp_InsightTimeStamp = -1;

function intrspDisplayInsight (data)
{
   var ins = $byId('insight');
   var stamp = new Date();
   stamp = stamp.getTime() / 1000;
   if (stamp - intrsp_InsightTimeStamp >= 1)
   {
      var  html = '';
      html += acmeIpcSummary();
      html += JSON.stringify(intrsp_StaticData,null,3);
      ins.innerHTML = html;
      intrsp_InsightTimeStamp = stamp;
      ins.setAttribute('ondblclick', 'intrspInsightClick()');
   }
   ins.innerHTML += '\n' + JSON.stringify(data,null,3);
   acmeAdjustSize();
}

///////////////////////////////////////////////////////////////////////////////

///////////////////////////
// manage intrusion data //
///////////////////////////

var intrsp_FirstTimestamp = 0,
    intrsp_CurrentRecordCount = 0;

var intrsp_IntrusionRecord = { 'order': [] };

function intrspRecordsReset ()
{
   intrsp_IntrusionRecord = { 'order': [] };
   intrsp_FirstTimestamp = intrsp_CurrentRecordCount = 0;
}

function intrspManageRecords ()
{
   if (!intrsp_IntrusionData &&
       !intrsp_CurrentRecordCount) return false;

   var count = intrsp_IntrusionData.recordCount;
   var records = intrsp_IntrusionData.records;
   var timestamp = intrsp_IntrusionData.timestamp;

   /*******************/
   /* current records */
   /*******************/

   intrsp_CurrentActiveCount = 0;

   for (var idx = 0; idx < count; idx++)
   {
      /* generate a unique key for this record */
      var ident = '', net = '', node = '', src = '';
      if (typeof records[idx].ident != 'undefined') ident = records[idx].ident;
      if (typeof records[idx].net != 'undefined') net = records[idx].net;
      if (typeof records[idx].node != 'undefined') node = records[idx].node;
      if (typeof records[idx].src != 'undefined') src = records[idx].src;
      var key = records[idx].class + '$' +
                ident + '$' + net + '$' + node + '$' + src;

      if (typeof intrsp_IntrusionRecord[key] == 'undefined') {
         intrsp_IntrusionRecord[key] = {};
         intrsp_IntrusionRecord.order.unshift(intrsp_IntrusionRecord[key]);
      }

      var the = intrsp_IntrusionRecord[key];

      the.timestamp = timestamp;
      the.expires = records[idx].expires;
      the.expiresTimestamp = records[idx].expiresTimestamp;
      the.historyTimestamp = timestamp + intrsp_HistorySecs;
      the.deleted = false;
      the.expired = false;

      if (typeof the.key == 'undefined')
      {
         /************************/
         /* new intrusion record */
         /************************/

         the.key = key;
         the.class = records[idx].class;
         the.count = records[idx].count;
         the.dns = records[idx].dns;
         the.ident = records[idx].ident;
         if (the.ident) the.ident = the.ident.replace("&","&amp;").replace(">","&gt;").replace("<","&lt;");
         the.net = records[idx].net;
         if (the.net) the.net = the.net.replace("&","&amp;").replace(">","&gt;").replace("<","&lt;");
         the.node = records[idx].node;
         the.src = records[idx].src;
         if (the.src) the.src = the.src.replace("&","&amp;").replace(">","&gt;").replace("<","&lt;");
         the.type = records[idx].type;
         the.lastType = the.type;

         if (typeof the.ident == 'undefined')
            the.ident = '<i>n/a</i>';

         if (typeof the.net == 'undefined')
            the.net = '<i>n/a</i>';

         if (typeof the.node == 'undefined')
            the.node = '<i>n/a</i>';

         if (typeof the.src == 'undefined')
            the.src = '<i>n/a</i>';

         the.delta = 0;
         the.prevCount = the.count;
         the.expiredCount = 0;

         the.wasExpired = false;
         the.deletedAlert = the.deltaAlert = the.expiredAlert =
            the.historyAlert = the.noticedAlert = false;
         the.alertTimestamp = 0;

         if (intrsp_FirstTimestamp)
         {
            /* not the first-ever time through */
            the.noticedTimestamp = timestamp;
            the.noticedAlert = true;
            the.noticed = intrsp_IntrusionData.vmsTime.split(' ')[1];
            the.alertTimestamp = timestamp + intrsp_NoticedSecs;
         }
         else
         {
            the.noticed = '<i>initial</i>';
            the.noticedTimestamp = 0;
            the.alertTimestamp = 0;
         }

         if (the.type.substr(0,1) == 'I')
            intrsp_TotalIntruderCount++;
         else 
            intrsp_TotalSuspectCount++;

         intrsp_TotalActiveCount++;
      }
      else
      {
         /*****************************/
         /* existing intrusion record */
         /*****************************/

         the.type = records[idx].type;

         if (the.wasExpired)
         {
            /* new intrusion after expiry */
            the.wasExpired = false;
            the.deletedAlert = the.deltaAlert = the.expiredAlert =
               the.historyAlert = false;
            the.noticedAlert = true;
            the.alertTimestamp = timestamp + intrsp_NoticedSecs;
            /* expired count allows a cumulative total to be kept */
            the.expiredCount = the.count;
            /* ensure this does not appear as a delta */
            the.prevCount = the.expiredCount + records[idx].count;
         }

         the.count = the.expiredCount + records[idx].count;
         if (the.count > the.prevCount)
         {
            the.delta += the.count - the.prevCount;
            the.alertTimestamp = timestamp + intrsp_DeltaSecs;
            the.deltaAlert = true;
         }
         the.prevCount = the.count;

         if (the.type != the.lastType)
         {
            /* suspect -> intruder is worth noting */
            the.lastType = the.type;
            if (the.type.substr(0,1) == 'I') intrsp_TotalIntruderCount++;
         }
      }

      the.intruderAlert = (the.type.substr(0,1) == 'I');
   }

   if (!intrsp_FirstTimestamp) intrsp_FirstTimestamp = timestamp;

   /***************/
   /* all records */
   /***************/

   intrsp_CurrentExpiredCount = 0;
   intrsp_CurrentDeletedCount = 0;
   intrsp_CurrentIntruderCount = 0;
   intrsp_CurrentRecordCount = 0;
   intrsp_CurrentSuspectCount = 0;

   for (var idx = 0; idx < intrsp_IntrusionRecord.order.length; idx++)
   {
      var the = intrsp_IntrusionRecord.order[idx];
      if (the == undefined) continue;

      if (intrsp_PurgeDisplay)
      {
         if (the.historyTimestamp < timestamp)
         {
            /* remove the record */
            intrsp_IntrusionRecord.order.splice(idx,1);
            delete intrsp_IntrusionRecord[the.key];
            continue;
         }
         else
         if (the.historyTimestamp - intrsp_FadingSecs < timestamp)
            the.historyAlert = true;
      }

      if (the.timestamp < timestamp &&
          the.expiresTimestamp > timestamp)
      {
         /* e.g. $ DELETE /INTRUSION * */
         the.deleted = true;
         the.deletedAlert = true;
         the.expiresTimestamp = 0;
         the.alertTimestamp = timestamp + intrsp_DeletedSecs;
         the.historyTimestamp = timestamp + intrsp_HistorySecs;
         intrsp_TotalDeletedCount++;
      }
   
      if (!the.expired &&
          the.expiresTimestamp - timestamp < 0)
      {
         intrsp_TotalExpiredCount++;
         the.expired = the.wasExpired = the.expiredAlert = true;
      }

      if (the.alertTimestamp &&
          the.alertTimestamp < timestamp)
      {
         /* alert has expired */
         the.alertTimestamp = 0;
         if (the.deltaAlert) the.delta = 0;
         the.deletedAlert = the.deltaAlert = the.expiredAlert =
            the.historyAlert = the.noticedAlert = false;
      }

      intrsp_CurrentRecordCount++;
      if (the.deleted) intrsp_CurrentDeletedCount++;
      if (the.expired) intrsp_CurrentExpiredCount++;
      if (!(the.deleted || the.expired))
      {
         intrsp_CurrentActiveCount++;
         if (the.type.substr(0,1) == 'I')
            intrsp_CurrentIntruderCount++;
         else
            intrsp_CurrentSuspectCount++;
      }
   }

   /*********************/
   /* accumulated total */
   /*********************/

   if (intrsp_CurrentActiveCount > intrsp_MaxActiveCount)
      intrsp_MaxActiveCount = intrsp_CurrentActiveCount;

   if (intrsp_CurrentIntruderCount > intrsp_MaxIntruderCount)
      intrsp_MaxIntruderCount = intrsp_CurrentIntruderCount;

   if (intrsp_CurrentDeletedCount > intrsp_MaxDeletedCount)
      intrsp_MaxDeletedCount = intrsp_CurrentDeletedCount;

   if (intrsp_CurrentExpiredCount > intrsp_MaxExpiredCount)
      intrsp_MaxExpiredCount = intrsp_CurrentExpiredCount;

   if (intrsp_CurrentRecordCount > intrsp_MaxRecordCount)
      intrsp_MaxRecordCount = intrsp_CurrentRecordCount;

   if (intrsp_CurrentSuspectCount > intrsp_MaxSuspectCount)
      intrsp_MaxSuspectCount = intrsp_CurrentSuspectCount;

   return (true);
}

///////////////////////////////////////////////////////////////////////////////

//////////////////////////
// display summary data //
//////////////////////////

/*
*/

var intrsp_CurrentActiveCount = 0,
    intrsp_CurrentDeletedCount = 0,
    intrsp_CurrentExpiredCount = 0,
    intrsp_CurrentIntruderCount = 0,
    intrsp_CurrentSuspectCount = 0,
    intrsp_MaxActiveCount = 0,
    intrsp_MaxDeletedCount = 0,
    intrsp_MaxExpiredCount = 0,
    intrsp_MaxIntruderCount = 0,
    intrsp_MaxRecordCount = 0,
    intrsp_MaxSuspectCount = 0,
    intrsp_TotalActiveCount = 0,
    intrsp_TotalDeletedCount = 0,
    intrsp_TotalExpiredCount = 0,
    intrsp_TotalIntruderCount = 0,
    intrsp_TotalSuspectCount = 0;

function intrspDisplayReset ()
{
   intrsp_CurrentActiveCount =
      intrsp_CurrentDeletedCount =
      intrsp_CurrentExpiredCount =
      intrsp_CurrentIntruderCount =
      intrsp_CurrentSuspectCount =
      intrsp_MaxActiveCount =
      intrsp_MaxDeletedCount =
      intrsp_MaxExpiredCount =
      intrsp_MaxIntruderCount =
      intrsp_MaxRecordCount =
      intrsp_MaxSuspectCount =
      intrsp_TotalActiveCount =
      intrsp_TotalDeletedCount =
      intrsp_TotalExpiredCount =
      intrsp_TotalIntruderCount =
      intrsp_TotalSuspectCount = 0;

   intrspCollectFor(true);
}

function intrspDisplaySummary ()
{
   if (intrsp_SynopsisDisplay)
   {
      intrspDisplaySynopsis();
      return;
   }

   if (!intrsp_IntrusionData) return;

   $byId('sumNodeName').innerHTML = intrsp_StaticData.nodeName;
   $byId('sumNodeName').setAttribute('onclick', 'intrspInsightClick()');
   var sumService = location.hostname;
   if (location.port.length) sumService += ':' + location.port;
   $byId('sumService').innerHTML = sumService;
   $byId('sumVMStime').innerHTML = intrsp_IntrusionData['vmsTime'];

   var html = '';
   html += intrsp_StaticData.hwName + ' with ' + intrsp_StaticData.cpuAvail +
           (intrsp_StaticData.cpuAvail > 1 ? ' CPUs and ' : ' CPU and ') +
           intrspValue(intrsp_StaticData.memSize) + 'B, running ' +
           intrsp_StaticData.archName + ' VMS ' + intrsp_StaticData.vmsVersion;

   html += '<br><span class="wordINTRUspect">INTRUspect</span>&nbsp;v' +
           $ExeVersion + ' under ' + intrsp_StaticData.HTTPd;

   html += ' collecting for ' + intrspCollectFor();

   html += '<br><b>' +
           intrsp_CurrentActiveCount + ' active</b> (' +
           intrsp_TotalActiveCount + ' total, ' +
           intrsp_MaxActiveCount + ' max), <b>' +
           intrsp_CurrentIntruderCount + ' intruder</b> (' +
           intrsp_TotalIntruderCount + ' total, ' +
           intrsp_MaxIntruderCount + ' max), <b>' +
           intrsp_CurrentSuspectCount + ' suspect</b> (' +
           intrsp_TotalSuspectCount + ' total, ' +
           intrsp_MaxSuspectCount + ' max)';

   if (intrsp_CurrentDeletedCount > 0)
      html += ', with ' + intrsp_CurrentDeletedCount + ' deleted (' +
              intrsp_MaxDeletedCount + ' max)';

   if (intrsp_CurrentExpiredCount > 0)
      html += ', and ' + intrsp_CurrentExpiredCount + ' expired (' +
              intrsp_MaxExpiredCount + ' max)';


   if (intrsp_IntrusionData.excessRecordCount)
      html += '<br><span style="color:red;">NOTE: ' +
              intrsp_IntrusionData.excessRecordCount +
              ' additional records not reported here!</span>';

   $byId('sumSystem').innerHTML = html;

   intrspDisplayRecords();

   acmeAdjustSize();
}

///////////////////////////////////////////////////////////////////////////////
/*
*/

function intrspDisplayRecords ()
{
   if (!intrspManageRecords()) return;

   if (!intrsp_IntrusionRecord.order.length)
   {
      $byId('intrusion').innerHTML = '<span id="zero">0 intrusion records</span>';
      return;
   }

   var timestamp = intrsp_IntrusionData.timestamp;

   // hmmm, collapse doesn't seem to work from .CSS file!
   var html =
'<div id="intruTableDiv">\
<table id="intruTable" style="border-collapse:collapse;">\
<tr><th></th>\
<th>Type</th>\
<th>Class</th>';

   if (intrsp_StaticData.clusterMember) html += '<th>Node</th>';

   html +=
'<th>Network</th>\
<th>Source</th>\
<th>Ident</th>\
<th style="text-align:right;">Count</th>\
<th style="text-align:right;">Delta</th>\
<th colspan="2">Noticed</th>\
<th colspan="2">Expires</th>\
</tr>';

   var count = 0;
   for (var idx = 0; idx < intrsp_IntrusionRecord.order.length; idx++)
   {
      var the = intrsp_IntrusionRecord.order[idx];
      if (the == undefined) continue;
      count++;

      if (intrsp_GeoLocateDisplay)
      {
         if (typeof the.dns != 'undefined' &&
             the.net.indexOf('.') > 0 &&
             the.net != '127.0.0.1')
            the.network = the.dns + ' (' + the.net + ') ' + geoLocate(the.net); 
         else
         if (the.net.indexOf('.') > 0 &&
             the.net != '127.0.0.1')
            the.network = the.net + ' ' + geoLocate(the.net); 
         else
            the.network = the.net; 
      }
      else
      if (typeof the.dns != 'undefined' && the.net.substr(0,1) != '<')
         the.network = the.dns + ' (' + the.net + ')';
      else
         the.network = the.net;
  
      var delta;
      if (the.noticedTimestamp)
         delta = timestamp - the.noticedTimestamp;
      else
         delta = timestamp - intrsp_FirstTimestamp;
      if (delta)
         the.noticedDelta = '&ndash;' + toHHMMSS(delta);
      else
         the.noticedDelta = '';

      the.expiryDelta = '';
      if (the.deleted)
         the.expires = 'DELETED';
      else
      if (the.expiresTimestamp - timestamp > 0)
         the.expiryDelta = '+' + toHHMMSS(the.expiresTimestamp -
                                          timestamp);
      else
      if (the.expiresTimestamp - timestamp < 0)
      {
         the.expiryDelta = '&ndash;' + toHHMMSS(the.expiresTimestamp -
                                                timestamp);
         if (the.expires.substr(0,1) != '<')
            the.expires = '<span style="text-decoration:line-through">' +
                          the.expires + '</span>';
      }

      if (the.deletedAlert)
         html += '<tr class="row_deleted">';
      else
      if (the.deltaAlert)
         html += '<tr class="row_delta">';
      else
      if (the.intruderAlert)
         html += '<tr class="row_intruder">';
      else
      if (the.historyAlert)
         html += '<tr class="row_history">';
      else
      if (the.expiredAlert)
         html += '<tr class="row_expired">';
      else
      if (the.noticedAlert)
         html += '<tr class="row_noticed">';
      else
         html += '<tr>';

      html += '<td style="min-width:2em;width:2em;text-align:right;">' + count +
              '.</td><td>' + the.type +
              '</td><td>' + the.class;

      if (intrsp_StaticData.clusterMember) 
         html += '</td><td>' + the.node;

      html += '</td><td class="thenetwork">' + the.network +
              '</td><td>' + the.src +
              '</td><td>' + the.ident +
              '</td><td style="text-align:right;">' + the.count +
              '</td><td style="text-align:right;">' + the.delta +
              '</td><td style="padding-right:0.5em;">' + the.noticed +
              '</td><td>' + the.noticedDelta +
              '</td><td style="padding-right:0.5em;">' + the.expires +
              '</td><td>' + the.expiryDelta +
              '</td></tr>';
   }

   html += '</table></div>';
   $byId('intrusion').innerHTML = html;
}

///////////////////////////////////////////////////////////////////////////////
/*
*/

function intrspDisplaySynopsis ()
{
   if (!intrsp_SynopsisDisplay) return;

   $byId('closedNodeName').innerHTML = intrsp_StaticData.nodeName;
   $byId('closedService').innerHTML = location.hostname + location.port;
   $byId('closedVMStime').innerHTML = intrsp_IntrusionData['vmsTime'];

   var html = ' &nbsp;&ndash;&nbsp; ';

   html += '<b>' +
           intrsp_CurrentActiveCount + ' active</b> (' +
           intrsp_TotalActiveCount + ' total, ' +
           intrsp_MaxActiveCount + ' max), <b>' +
           intrsp_CurrentIntruderCount + ' intruder</b> (' +
           intrsp_TotalIntruderCount + ' total, ' +
           intrsp_MaxIntruderCount + ' max), <b>' +
           intrsp_CurrentSuspectCount + ' suspect</b> (' +
           intrsp_TotalSuspectCount + ' total, ' +
           intrsp_MaxSuspectCount + ' max)';

   $byId('closedSynopsis').innerHTML = html;
}

//////////////////////////////////////////////////////////////////////////////
/*
Return how long INTRUspect has been collecting for.
*/

var intrsp_CollectLastFor = '',
    intrsp_CollectStartTime = new Date();

function intrspCollectFor (zero)
{
   if (typeof zero != 'undefined' && zero) intrsp_CollectStartTime = new Date();
   if (!intrsp_CollectDisplay) return intrsp_CollectLastFor;

   var now = new Date();
   var then = intrsp_CollectStartTime;
   var diff = (Math.abs((now.getTime() - then.getTime()) / 1000)).toFixed(0);
   var secs = Math.floor(diff % 60).toFixed(0);
   diff /= 60;
   var mins = Math.floor(diff % 60).toFixed(0);
   diff /= 60;
   var hours = Math.floor(diff % 24).toFixed(0);
   diff /= 24;
   var days = Math.floor(diff).toFixed(0);
   if (hours < 10) hours = '0' + hours;
   if (mins < 10) mins = '0' + mins;
   if (secs < 10) secs = '0' + secs;
   intrsp_CollectLastFor = '';
   if (parseInt(days) > 0) intrsp_CollectLastFor = days + ' ';
   intrsp_CollectLastFor += hours + ':' + mins + ':' + secs;
   return intrsp_CollectLastFor;
}

///////////////////////////////////////////////////////////////////////////////
/*
Return a string respresenting the value paramater transmogrified to kilos and
megas as seem more meaningful.  Kilo == 1024!
*/

function intrspValue (value,kat)
{
   // useful during development :-)
   if (typeof value == 'undefined') return '?';

   if (typeof kat == 'undefined') kat = 10;

   var fixed = 0;
   if (value > 1099511627776)
   {
      if (value > 1099511627776*100)
         fixed = 1;
      else
      if (value > 1099511627776*10)
         fixed = 2;
      else
         fixed = 3;
      var string = ((value/1099511627776).toFixed(fixed)-0).toString() + 'T';
   }
   else
   if (value > 1073741824)
   {
      if (value > 1073741824*100)
         fixed = 1;
      else
      if (value > 1073741824*10)
         fixed = 2;
      else
         fixed = 3;
      var string = ((value/1073741824).toFixed(fixed)-0).toString() + 'G';
   }
   else
   if (value > 1048576)
   {
      if (value > 1048576*100)
         fixed = 1;
      else
      if (value > 1048576*10)
         fixed = 2;
      else
         fixed = 3;
      var string = ((value/1048576).toFixed(fixed)-0).toString() + 'M';
   }
   else
   if (value > 1024*kat)
   {
      if (value <= 1024*100) fixed = 1;
      var string = ((value/1024).toFixed(fixed)-0).toString() + 'k';
   }
   else
      var string = value.toFixed(0);

   return string;
}

///////////////////////////////////////////////////////////////////////////////

function intrspWithCommas(num)
{
   return num.toString().replace(/\B(?=(\d{3})+(?!\d))/g, intrsp_ThousandComma);
}

///////////////////////////////////////////////////////////////////////////////
/*
s or ss or mm:ss or hh:mm:ss
*/

function toHHMMSS(seconds)
{
   if (seconds < 0) seconds *= -1;
   var sec_num = parseInt(seconds);
   var hours = Math.floor(sec_num / 3600);
   var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
   var seconds = sec_num - (hours * 3600) - (minutes * 60);        

   if (hours == 0)
      hours = '';
   else
   if (hours < 10)
      hours = '0'+hours;

   if (minutes == 0 && hours > 0)
      minutes = '00';
   else
   if (minutes == 0)
      minutes = '';
   else
   if (minutes < 10)
      minutes = '0' + minutes;
   if (hours > 0) minutes = ':' + minutes;

   if (seconds == 0 && (hours > 0 || minutes > 0))
      seconds = '00';
   else
   if (seconds < 10 && (hours > 0 || minutes > 0))
      seconds = '0' + seconds;
   else
   if (seconds == 0)
      seconds = '0';
   if (hours > 0 || minutes > 0) seconds = ':' + seconds;

   var time = hours + minutes + seconds;
   return time;
}  

///////////////////////////////////////////////////////////////////////////////

